
{
  open Lexing
  open Parser
  exception Lexical_error of string

  let id_or_keyword =
    let h = Hashtbl.create 17 in
    List.iter (fun (s,k) -> Hashtbl.add h s k)
      [ 
	"Definition", DEFINITION;
	"Inductive", INDUCTIVE;
	"Parameter", PARAMETER;
	"Prop", PROP; 
	"Type", TYPE;
	"as", AS; 
	"end", END; 
	"fix", FIX; 
	"forall", FORALL; 
	"fun", FUN; 
	"in", IN; 
	"match", MATCH; 
	"return", RETURN;
	"with", WITH; 
      ];
    fun s -> 
      try Hashtbl.find h s with Not_found -> IDENT s

  let newline lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <- 
      { pos with pos_lnum = pos.pos_lnum + 1; pos_bol = pos.pos_cnum; 
        pos_cnum=0 }

}

let alpha = ['a'-'z' 'A'-'Z']
let digit = ['0'-'9']
let ident = alpha (alpha | '_' | digit | '\'')*

rule token = parse
  | '\n' 
      { newline lexbuf; token lexbuf }
  | [' ' '\t' '\r']+ 
      { token lexbuf }
  | "(*" 
      { comment lexbuf; token lexbuf }
  | ident  
      { id_or_keyword (lexeme lexbuf) }
  | digit+
      { INTEGER (int_of_string (lexeme lexbuf)) }
  | '('
      { LPAR }
  | ')'
      { RPAR }
  | '|'
      { BAR }
  | ':'
      { COLON }
  | ','
      { COMMA }
  | '.'
      { DOT }
  | "->"
      { ARROW }
  | "=>"
      { BIGARROW }
  | ":=" 
      { COLONEQUAL }
  | _ 
      { raise (Lexical_error ("illegal character: " ^ lexeme lexbuf)) }
  | eof  
      { EOF }

and comment = parse
  | "*)" { () }
  | "(*" { comment lexbuf; comment lexbuf }
  | '\n' { newline lexbuf; comment lexbuf }
  | _    { comment lexbuf }
  | eof  { raise (Lexical_error "unterminated comment") }

