
open Format
open Lexing
open Lexer
open Parser

let usage () = eprintf "usage: min_coq file.v@."; exit 2
let () = if Array.length Sys.argv <> 2 then usage ()
let file = Sys.argv.(1)
let () = 
  if not (Sys.file_exists file) then begin
    eprintf "mini_coq: %s: no such file@." file; exit 2
  end
  
let report_loc (b,e) =
  let l = b.pos_lnum in
  let fc = b.pos_cnum - b.pos_bol + 1 in
  let lc = e.pos_cnum - b.pos_bol + 1 in
  eprintf "File \"%s\", line %d, characters %d-%d:\n" file l fc lc

let () =
  let c = open_in file in
  let lb = Lexing.from_channel c in
  try
    let _ = Parser.file Lexer.token lb in
    ()
  with
    | Lexer.Lexical_error s -> 
	report_loc (lexeme_start_p lb, lexeme_end_p lb);
	eprintf "lexical error: %s@." s;
	exit 1
    | Parsing.Parse_error ->
	report_loc (lexeme_start_p lb, lexeme_end_p lb);
	eprintf "syntax error@.";
	exit 1
    | e ->
	eprintf "unexpected error: %s@." (Printexc.to_string e);
	exit 2

    
