
%{
  open Ast
  let loc () = symbol_start_pos (), symbol_end_pos ()
  let locate e = { node = e; loc = loc () }
%}

%token <string> IDENT
%token <int> INTEGER
%token LPAR RPAR BAR COLON COMMA COLONEQUAL EOF DOT ARROW BIGARROW
%token FORALL FUN MATCH FIX WITH IN END AS RETURN PROP TYPE
%token DEFINITION INDUCTIVE PARAMETER

%type <Ast.file> file
%start file

%nonassoc COMMA BIGARROW FUN FORALL FIX prec_fix
%right ARROW
%nonassoc IDENT LPAR PROP TYPE MATCH
%left prec_app

%%

file:
  list0_decl EOF { $1 }
;

list0_decl:
  /* epsilon */   { [] }
| decl list0_decl { $1 :: $2 }
;

decl:
  DEFINITION IDENT COLON constr COLONEQUAL constr DOT 
    { locate (Definition ($2, $4, $6)) }
| PARAMETER IDENT COLON constr DOT 
    { locate (Parameter ($2, $4)) }
| INDUCTIVE IDENT list0_closed_binder COLON constr COLONEQUAL 
  list0_constructor DOT
    { locate (Inductive ($2, $3, $5, $7)) }
;

constr:
  IDENT 
    { locate (Cident $1) }
| PROP  
    { locate Cprop }
| TYPE  
    { locate (Ctype 1) }
| TYPE INTEGER 
    { locate (Ctype $2) }
| LPAR constr RPAR 
    { $2 }
| constr constr %prec prec_app 
    { locate (Capp ($1, $2)) }
| constr ARROW constr
    { locate (Carrow ($1, $3)) }
| FORALL binders COMMA constr
    { locate (Cforall ($2, $4)) }
| FUN binders BIGARROW constr
    { locate (Cfun ($2, $4)) }
| MATCH constr opt_as_in RETURN constr WITH list0_branch END
    { locate (Cmatch ($2, $3, $5, $7)) }
| FIX IDENT list1_closed_binder COLON constr COLONEQUAL constr %prec prec_fix
    { locate (Cfix ($2, $3, $5, $7)) }
;

binders:
  list1_ident COLON constr { [$1,$3] }
| list1_closed_binder      { $1 }
;

list0_closed_binder:
  /* epsilon */       { [] }
| list1_closed_binder { $1 }
;

list1_closed_binder:
  closed_binder                     { [$1] }
| closed_binder list1_closed_binder { $1 :: $2 }
;

closed_binder:
  LPAR list1_ident COLON constr RPAR { $2,$4 }
;

list1_ident:
  IDENT             { [locate $1] }
| IDENT list1_ident { locate $1 :: $2 }
;

opt_as_in:
  /* epsilon */      { None }
| AS IDENT IN constr { Some ($2, $4) }
;

list0_branch:
  /* epsilon */       { [] }
| branch list0_branch { $1 :: $2 }
;

branch:
  BAR list1_ident BIGARROW constr { $2, $4 }
;

list0_constructor:
  /* epsilon */                 { [] }
| constructor list0_constructor { $1 :: $2 }
;

constructor:
  BAR ident COLON constr { $2, $4 }
;

ident:
  IDENT { locate $1 }
;

